*  Move Block .01
*  By Adam Trionfo, based on a program by Eric Beckett.
*  July 14, 2010
*  
*  This is a re-written version of the Move Square program by Eric 
*  Beckett from the Aug, Sept, and Nov, 1984 issues of the APF
*  Imagination  Machine Newletter, "I-M 1 in a Million."  That
*  version required BASIC.  This version runs as a cartridge.
*  
*  This program is written to be run on the circa 1978 APF
*  MP-1000 computer game console which uses the
*  Motorola 6800 8-Bit CPU. 
*      
*  Program Description
*  -------------------  
*
*  The APF Menu gives one choice to the user: 1) RUN PROGRAM
*  
*  This program places a white block in the middle of the screen.
*  The user can control it using the Right Joystick.  The four
*  cardinal directions are supported-- diagnals are NOT supported.
*  
*  Notes:
*  ------
*  There is NO boundry checking of the White Block.  Moving the
*  cursor left or right will cause the Block to "wrap."  If the
*  White Block is moved up or down off the screen, then it is 
*  possible to crash the program because the Block is moving 
*  into RAM.
*  
*  Version
*  -------
*  .01 - First Release  

*  Equates
    
FILLSCRN  EQU  $4296        ; Fill Screen Routine
MIDSCR    EQU  $02EE        ; Location of Middle of Screen
CURLOC    EQU  $0180        ; Current Location of the White Block
JOYDATA   EQU  $01F2        ; Joystick Data
READJOY2  EQU  $41D9        ; Read Right Joystick
NORTH     EQU  $4E          ; ASCII Capital N
SOUTH     EQU  $53          ; ASCII Capital S
EAST      EQU  $45          ; ASCII Capital E
WEST      EQU  $57          ; ASCII Capital W
BACKGRND  EQU  $80          ; ASCII of Background
WHITEBLK  EQU  $CF          ; ASCII of White Block
          
          ORG   $8000       ; Start of APF Cartridge Area
          
*  The BIOS uses the first five bytes of the cartridge  

          FCB   $BB         ; Tell BIOS a cart is present
          FDB   MENUSTR     ; Points to Menu string on cartridge 
          FCB   $31         ; LSB 4 bits is # of Menu Choices
*                             Menu has one choice          
          FCB   $00         ; Checksum-type byte.  Must be $00 else
*                             BIOS restarts right after Init

*  After a menu choice is made from the APF Menu via the
*  BIOS, then control passes to the program here ($8005).  
 
          JSR   FILLSCRN    ; Fill Screen Background with Black      

**********************          
*                    *
*  Main Program Loop *
*                    *
**********************

          JSR   INITPROG    ; Initialize Program
MOVBLOCK  JSR   READSTIK    ; Read Joystick and Move Block    
          JMP   MOVBLOCK    ; Continue Running Program Forever 

*****************          
*               *
*  Subroutines  *
*               *
*****************

* Initialize Program
* ------------------
* Move the White Block to the middle of the screen.

INITPROG  LDX   #MIDSCR     ; Middle of Screen
          STX   CURLOC      ; Current location of the White Block
          LDAA  #WHITEBLK   ; Place ASCII White Block into Accum A 
          STAA  $00,X       ; Store White Block on Screen
          RTS

*  Has the Joystick been moved?
*  ----------------------------

READSTIK  JSR   READJOY2    ; Right Joystick Subroutine 
          BCS   JOY2USED    ; Branch if Joystick Used
          RTS               ; No Movement- Return and keep checking

*  Check which direction the joystick has been moved
*  -------------------------------------------------

* Check North
JOY2USED  LDAA  JOYDATA     ; Put Joystick Direction Data into Accum A 

          CMPA  #NORTH      ; Did Joystick Move North?
          BNE   CHKSOUTH    ; If not, then Check South Movement
          JMP   MOVNORTH    ; Direction was North, Branch

* Check South 
CHKSOUTH  CMPA  #SOUTH      ; Did Joystick Move South? 
          BNE   CHKEAST     ; If not, then Check East Movement
          JMP   MOVSOUTH    ; Direction was South, Branch

* Check East 
CHKEAST   CMPA  #EAST       ; Did Joystick Move East?
          BNE   CHKWEST     ; If not, then Check West Movement
          JMP   MOVEAST     ; Direction was East, Branch

* Check West
CHKWEST   CMPA  #WEST       ; Did Joystick Move West?
          BNE   NODIR       ; If not, then No Direction was Picked
          JMP   MOVWEST     ; Direction was West, Branch

NODIR     RTS               ; No Movement- Return and keep checking 

* Update White Block's Movement on the Screen
* -------------------------------------------

* White Block's East Movement
MOVEAST   LDX   CURLOC      ; Load Current Location of White Block
          LDAA  #BACKGRND   ; Load Background Color into Accum A
          STAA  $00,X       ; Store Background Color onto screen
          INX               ; Move Block Location East
          JMP   DRAWBLOK    ; Jump to Draw White Block Subroutine

* White Block's West Movement 
MOVWEST   LDX   CURLOC      ; Load Current Location of White Block
          LDAA  #BACKGRND   ; Load Background Color into Accum A
          STAA  $00,X       ; Store Background Color onto screen
          DEX               ; Move Block Location West
          JMP   DRAWBLOK    ; Jump to Draw White Block Subroutine

* White Block's North Movement
MOVNORTH  LDX   CURLOC      ; Load Current Location of White Block
          LDAA  #BACKGRND   ; Load Background Color into Accum A
          STAA  $00,X       ; Store Background Color onto screen
          LDAA  #32D        ; Initialize Loop for 32 Times
BLOCKUP   DEX               ; Countdown Loop for "Up" Movement 
          DECA              ; Decrement "Up" Movement Loop 
          BNE   BLOCKUP     ; Has screen RAM moved "Up"
          JMP   DRAWBLOK    ; Jump to Draw White Block Subroutine

* White Block's South Movement
MOVSOUTH  LDX   CURLOC      ; Load Current Location of White Block
          LDAA  #BACKGRND   ; Load Background Color into Accum A
          STAA  $00,X       ; Store Background Color onto screen
          LDAA  #32D        ; Initialize Loop for 32 Times 
BLOKDOWN  INX               ; Countdown Loop for "Down" Movement 
          DECA              ; Decrement "Down" Movement Loop
          BNE   BLOKDOWN    ; Has screen RAM moved "Down"

* Draw White Block - Draw Block at New Locaction.
* ----------------
DRAWBLOK          
          LDAA  #WHITEBLK   ; Place ASCII White Block into Accum A
          STAA  $00,X       ; Draw White Block to Screen
          STX   CURLOC      ; Update Current Location of White Block

*  Delay Loop 
*  ----------
* 
*  This loop determines the speed of the block.

DELYLOOP  LDAB  #50D         ; Number of Outer Loops 
OUTLOOP   LDAA  #255D        ; Number of Inner Loops
INLOOP    DECA               ; Decrement Inner Loop
          BNE   INLOOP       ; All Inner Loops Complete?   
          DECB               ; Decrement Outer Loop
          BNE   OUTLOOP      ; All Outer Loops Complete?
          
          RTS                ; Check for More Joystick Movement
 
*  Cartridge Menu String
*  ---------------------
*  
*  The data required by the BIOS for the main menu is here.

MENUSTR   FCB   $E9         ; 9 spaces before Program Name string
          FCC   "MOVE BLOCK  .01"  ; Program Name String
          FCB   $E8         ; 8 spaces after text (EOL)
          FCB   $E1         ; 1 spaces (Before Author)
          FCC   "BY ADAM TRIONFO, JULY 14, 2010"  ; Author
          FCB   $E3         ; 3 spaces (After Author)
          FCB   $E3         ; 3 spaces (Before Author)          
          FCC    "BASED ON ERIC BECKETT'S"
          FCB   $E7         ; 7 spaces (After Author)
          FCC    "BASIC/ML EXAMPLE FROM 1984"
          FCB   $FE         ; Another 30 spaces
          FCB   $E5         ; Another 5 spaces
          FCC   "1. RUN PROGRAM                  "
          FCB   $FF         ; Control Byte - End of String

CARTEND   FILL  $FF,$9000-CARTEND ; Pad with $FF for a 4K Cart

* End of Program
